/* ***** BEGIN LICENSE BLOCK *****
 *   Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is Save File to.
 *
 * The Initial Developer of the Original Code is
 * Atte Kemppilä.
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 * 
 * ***** END LICENSE BLOCK ***** */


// nsIController
function TreeController(treeView)
{
    this.treeView = treeView;
}

TreeController.prototype =
{
    commands: "copy cut paste selectAll".split(" "),
    
    get commandSet()
    {
        var prototype = TreeController.prototype;
        var commandSet = new Set(prototype.commands);
        return lazyInit(prototype, "commandSet", commandSet);
    },
    
    isCommandEnabled: function(command)
    {
        var treeView = this.treeView;
        var selection = treeView.selection;
        
        switch (command)
        {
            case "cmd_copy":
                return (selection.count > 0);
                
            case "cmd_cut":
                return (selection.count > 0);
                
            case "cmd_paste":
                return (this._getClipboardEntries().length > 0);
                
            case "cmd_selectAll":
                return true;
                
            default:
                throw new Error("invalid command: " + command);
        }
    },
    
    supportsCommand: function(command)
    {
        // strip "cmd_"
        command = command.substr(4);
        return (command in this.commandSet);
    },
    
    doCommand: function(command)
    {
        if (this.isCommandEnabled(command))        
            this[command]();
    },
    
    onEvent: function(eventName) { /* empty */ },
    
    cmd_copy: function()
    {
        var treeView = this.treeView;
        var entries = treeView.selectedEntries;
        var transferable = this._createTransferable(entries);
        clipboard.setData(transferable, null, Ci.nsIClipboard.kGlobalClipboard);
    },
    
    cmd_cut: function()
    {
        var treeView = this.treeView;
        this.cmd_copy();
        treeView.removeEntries(treeView.selectedIndexes);
    },
    
    cmd_paste: function()
    {
        var treeView = this.treeView;
        var index = treeView.selection.currentIndex;
        var entries = this._getClipboardEntries();
        treeView.addEntries(index, entries, false);
    },
    
    cmd_selectAll: function()
    {
        var treeView = this.treeView;
        treeView.selection.selectAll();
    },
    
    _getClipboardEntries: function()
    {
        var transferable = Cc["@mozilla.org/widget/transferable;1"].createInstance(Ci.nsITransferable);
        transferable.addDataFlavor(FLAVOR_SAVEFILETO_ENTRIES);
        transferable.addDataFlavor("text/unicode");
        
        clipboard.getData(transferable, Ci.nsIClipboard.kGlobalClipboard);
        var flavor = {}, data = {}, length = {};
        
        try       { transferable.getAnyTransferData(flavor, data, length); }
        catch (e) { return []; }
        
        var entries = [];
        var file, path;
        
        switch (flavor.value)
        {
            case FLAVOR_SAVEFILETO_ENTRIES:
                var json = data.value.QueryInterface(Ci.nsISupportsString).data;
                entries = JSON.parse(json);
                break;
                
            case "text/unicode":
                path = data.value.QueryInterface(Ci.nsISupportsString).data;
                break;
                
            default:
                throw new Error("invalid flavor: " + flavor.value);
        }
        
        if (path)
        {
            try
            {
                var f = Cc["@mozilla.org/file/local;1"].createInstance(Ci.nsILocalFile);
                f.initWithPath(path);
                file = f;
            }
            catch (e) { /* ignore */ }
        }
        
        if (file && file.exists() && file.isDirectory())
            entries = [{type: "folder", label: file.leafName, path: file.path, menu: true}];
        
        return entries;
    },
    
    _createTransferable: function(entries)
    {
        var transferable = Cc["@mozilla.org/widget/transferable;1"].createInstance(Ci.nsITransferable);
        
        function addData(flavor, value)
        {
            var str = Cc["@mozilla.org/supports-string;1"].createInstance(Ci.nsISupportsString);
            str.data = value;
            transferable.addDataFlavor(flavor);
            transferable.setTransferData(flavor, str, value.length * 2);
        }
        
        var json = JSON.stringify(entries);
        addData(FLAVOR_SAVEFILETO_ENTRIES, json);
        
        var isFolder = function(entry) { return (entry.type == "folder"); }
        if (entries.every(isFolder))
            addData("text/unicode", [entry.path for each (entry in entries)].join("\n"));
        
        return transferable;
    }
}
